# README: If you run this app locally, please set the "App" folder as local working directory
#
# This is a Shiny web application. You can run the application by clicking
# the 'Run App' button above.
#
# Find out more about building applications with Shiny here:
#
#    http://shiny.rstudio.com/
#

# Hinzufügen: Linie für Soll, Fehlerbereich mit grauen Balken, option, dass der plot bei null beginnt und nicht immer mittendrin ?
# Farblich kennzeichnen welche Proben mit alter, neuer MS und OES gemessen wurden.
# MS und OES Linien nebeneinander.
# Other samples: ProjNr in Tooltip
# ggplot : grauer Hintergrund weg.



library(shiny)
library(data.table)
library(plotly)
library(dplyr)


dataset <- readRDS("./data/Standards.rds") # oder relativen Pfad zum Hauptort des files..
Std_ref <- readRDS("./data/STD_Reference.rds")

ui <- fluidPage(

   # Application title
   titlePanel("Display the fit of the standards with graphics"),

   # select input
   sidebarLayout(
      sidebarPanel(
        h4("Reference materials"),
        selectInput("var",
                    label = "Choose from these:",
                    choices = list(
                    "BCR- 129 (hay powder)",
                    "Clayey Shale 59",
                    "Clayey Shale 45",
                    "NJV 94-5 (wood fuel)" ,
                    "GBW 10052 (Tea)",
                    "NCS DC 73349 (bush leaves)" ,
                    "WEPAL- IPE- 126" ,
                    "WEPAL- IPE- 168" ,
                    "JLk- 1 (lake sediment)" ,
                    "JA-2",
                    "BB-IGDL (Basalt)" ), selected = "WEPAL- IPE- 126"),

        htmlOutput("selectUI"),

        selectInput("ele",
                    label = "Which element should be displayed ?",
                    choices = list("Al","As","B","Ba","Be","Bi","Ca","Cd","Ce","Co","Cr","Cs","Cu","Dy","Er","Eu","Fe","Ga","Gd","Ge","Hf","Ho","Ir","K","La","Li","Lu","Mg","Mn","Mo","Na","Nb",
                                   "Nd","Ni","P","Pb","Pr","Pt","Rb","S","Sb","Sc","Se","Sm","Sn","Sr","Ta","Tb","Te","Th","Ti","Tl","Tm","U","V","W","Y","Yb","Zn","Zr"), selected = "Co"),

        wellPanel(

        radioButtons("plot_type", "Choose type of Plot",
                     c("plotly", "ggplotly"))
        )
          ),

      mainPanel(
         plotlyOutput("plot"),
         br(),
        tags$p("The certified value (mg/kg) is at: "),
        textOutput("derWert")
         #verbatimTextOutput(outputId = "derWert")
      )
   ))



server <- function(input, output, session) {

            output$selectUI <- renderUI({
              selectInput("dates", label = "Choose date of measurement (OES), but MS is included",
                          choices = dates_subset())
            })

            dataset_subset <- reactive({
             a <- data.table(subset(dataset, Species == input$var & Date.OES == input$dates))

             col_el <- names(a)[grep(paste(input$ele,"[^a-zA-Z]", sep=""), names(a))]
             c <- data.table(a[,c("ProjNr","Species", "Project" ,"Series", col_el), with=F])

             df_melt<- melt(c, measure.vars = col_el)
            return(df_melt)

            })

            cert_val <- reactive({
              b <- as.numeric(subset(Std_ref, Species == input$var)[, input$ele, with=F])

              return(b)
              })

            dates_subset <- reactive({
              df2 <- unique(sort(data.table(subset(dataset, Species == input$var))[,Date.OES]))
              return(df2)
            })

            dataset_other_sampl <- reactive( {
              df1 <- data.table(subset(dataset, Species == input$var))

              col_el <- names(df1)[grep(paste(input$ele,"[^a-zA-Z]", sep=""), names(df1))]
              df2 <- data.table(df1[,c("ProjNr","Species", "Project" ,"Series","Date.OES", "Date.MS", col_el), with=F])
              df2_melt<- melt(df2, measure.vars = col_el)
              return(df2_melt)
            })


             output$plot <- renderPlotly({

               if(input$plot_type == "plotly"){
               plot_ly(data = dataset_other_sampl(), y= ~ value, x= ~variable, type="scatter", mode="markers",
                  name = "other samples",
                  text = ~paste("ProjNr:",ProjNr))%>%
                 add_markers(data = dataset_subset(), symbol = I(1),y= ~value , x=~ variable, name = "chosen samples",
                           marker = list(color = ~"red", line = list(color = "black", width = 2), size=10), type="scatter",
                           opacity=0.9,
                           text = ~paste("ProjNr:",ProjNr))%>%

              add_trace(y= ~cert_val(),  mode = 'lines', name = "cert. value") %>%
               layout(title = input$var)

                 } else if (input$plot_type =="ggplotly"){

              p = ggplot(data=dataset_other_sampl(), aes(x= variable, y = value))+
                    theme_bw() +
                    geom_point(aes(col="other"))+
                    geom_hline (yintercept = cert_val(),col="red")+
                    geom_point(data=dataset_subset(), aes(x= variable, y = value, col=ProjNr))+
                    ggtitle(input$var)+
                    theme(axis.text.x = element_text(angle=45, hjust=1),
                          axis.title.x = element_blank())
              print (ggplotly(p))
                 }
             })

               output$derWert <- renderText(cert_val())

           }


# Run the application
shinyApp(ui = ui, server = server)

